/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Staging repositories mediator.
 *
 * @since staging 2.3
 */
NX.define('Nexus.staging.repository.RepositoriesMediator', {
    extend: 'Ext.util.Observable',
    singleton: true,

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.profile.ProfileStore',
        'Nexus.staging.repository.RepositoryStore',
        'Nexus.staging.rule.RuleSetStore',
        'Nexus.staging.rule.RuleTypeStore',
        'Nexus.staging.repository.ConfirmWindow'
    ],

    /**
     * @constructor
     */
    constructor: function () {
        this.profileStore = NX.create('Nexus.staging.profile.ProfileStore');
        this.repositoryStore = NX.create('Nexus.staging.repository.RepositoryStore');
        this.ruleSetStore = NX.create('Nexus.staging.rule.RuleSetStore');
        this.ruleTypeStore = NX.create('Nexus.staging.rule.RuleTypeStore');

        // provided centralized event management allowing for dynamic enhancement of UI's depending on this class
        this.addEvents(
            /**
             * Fires when repositoriesGrid columns are being defined.
             *
             * @event staging.repositoriesGrid.configureColumns
             * @param {Array} Column configuration objects for Ext.grid.ColumnModel.
             */
            'staging.repositoriesGrid.configureColumns',

            /**
             * Fires when SummaryView properties are being defined.
             *
             * @event staging.repositorySummary.configureProperties
             * @param {Array} properties Objects with name/value properties; push new Object to extend the view.
             * @param {Array} values Repository data for inspection.
             */
            'staging.repositorySummary.configureProperties',

            // NOTE: These are not really in the right place, these are profile specific and this mediator is repository specific

            /**
             * Fires when StagingProfile is submitted.
             *
             * @event staging.profile.submitted
             * @param {Array} properties The StagingProfile properties; create an entry with the KEY for a Field created by staging.profile.loaded event to
             * include it in the submitted form.
             */
            'staging.profile.submitted',

            /**
             * Fires when StagingProfile is loaded.
             *
             * @event staging.profile.loaded
             * @param {MixedCollection} items The Ext.form.Fields definitions for the StagingProfile form; push new Field to extend the view.
             * @param {Array} properties The StagingProfile properties for inspection.
             * @param {String} dataRoot The prefix to be used for naming new Fields. Added fields should define {name: dataRoot + '.' + KEY}
             */
            'staging.profile.loaded'
        );
    },

    refreshHandler: function () {
        this.logDebug('Refreshing');

        this.profileStore.reload();
        this.repositoryStore.reload();
        this.ruleSetStore.reload();
        this.ruleTypeStore.reload();
    },

    closeHandler: function (selected) {
        this.displayConfirmWindow('close', selected);
    },

    dropHandler: function (selected) {
        this.displayConfirmWindow('drop', selected);
    },

    releaseHandler: function (selected) {
        this.displayConfirmWindow('release', selected);
    },

    promoteHandler: function (selected) {
        this.displayConfirmWindow('promote', selected);
    },

    /**
     * @private
     */
    displayConfirmWindow: function (actionName, selected) {
        var self = this,
            action = self.getActionWords(actionName),
            selectedIds = self.repositoryIdsForSelection(selected),
            dialog;

        dialog = NX.create('Nexus.staging.repository.ConfirmWindow', {
            activity: actionName,
            url: Sonatype.config.servicePath + '/staging/bulk/' + action.rest,
            message: '<p>You are about to <b>' + action.verb + '</b> ' +
                Ext.util.Format.plural(selectedIds.length, 'repository', 'repositories') + '.</p>' +
                '<p>This may take some time to complete.</p>',

            // HACK: Test adding a custom field item
            customizeFormItems: function (items) {
                // release needs to include a checkbox to allow automatic dropping
                if (actionName === 'release') {
                    items.push({
                        xtype: 'fieldset',
                        title: 'Options',
                        labelSeparator: '',
                        items: [
                            {
                                xtype: 'checkbox',
                                name: 'autoDropAfterRelease',
                                fieldLabel: 'Automatically Drop',
                                helpText: 'After the release operation is successful, automatically drop released repositories.  Automatic drop was the default behavior of Nexus 2.3 and earlier.',
                                checked: true
                            }
                        ]
                    });
                }
                else if (actionName === 'promote') {
                    // promotion needs to include a promotion profile selection
                    items.push({
                        xtype: 'fieldset',
                        title: 'Options',
                        labelSeparator: '',
                        items: [
                            {
                                xtype: 'combo',
                                name: 'stagingProfileGroup',
                                fieldLabel: 'Promotion Profile',
                                emptyText: 'Select a profile',

                                mode: 'local',
                                store: self.createPromotionProfileStore(selected),
                                displayField: 'name',
                                valueField: 'id', // FIXME: This is not working, value is always from 'name' field :-(
                                triggerAction: 'all',

                                forceSelection: true,
                                editable: false,
                                selectOnFocus: true,
                                allowBlank: false,
                                required: true
                            }
                        ]});
                }
            },

            customizeFormData: function (form, values) {
                values.stagedRepositoryIds = selectedIds;

                // HACK: Work around with checkbox setting form value to "on" instead of true (boolean value)
                if (actionName === 'release') {
                    values.autoDropAfterRelease = form.findField('autoDropAfterRelease').getValue();
                }

                // HACK: Work around issue with combobox not returning valueField
                if (actionName === 'promote') {
                    values.stagingProfileGroup = form.findField('stagingProfileGroup').getValue();
                }

                return {
                    data: values
                }
            },

            listeners: {
                success: function (window, response, options) {
                    self.displaySuccess(action.Noun + ' ' + Ext.util.Format.plural(selectedIds.length, 'repository', 'repositories'));
                },

                failure: function (window, response, options) {
                    self.displayFailure(action.Verb + ' failed', response);
                }
            }
        });

        dialog.show();
    },

    /**
     * @private
     */
    repositoryIdsForSelection: function (selected) {
        var result = [];

        Ext.each(selected, function (record) {
            result.push(record.data.repositoryId);
        });

        return result;
    },

    /**
     * @private
     */
    actionWords: {
        close: {
            noun: 'closing',
            verb: 'close'
        },

        drop: {
            noun: 'dropping',
            verb: 'drop'
        },

        promote: {
            noun: 'promoting',
            verb: 'promote'
        },

        release: {
            noun: 'releasing',
            verb: 'release',
            rest: 'promote'
        }
    },

    /**
     * @private
     */
    getActionWords: function (actionName) {
        var action = this.actionWords[actionName];

        // auto capitalize
        if (!action.capitalized) {
            action.Noun = Ext.util.Format.capitalize(action.noun);
            action.Verb = Ext.util.Format.capitalize(action.verb);
            action.capitalized = true;
        }

        // auto sort rest shit, the names used here don't line up anymore so we fux around it
        if (!action.rest) {
            action.rest = action.verb;
        }

        return action;
    },

    /**
     * Build the promotion profile store, which is displayed to the user in the promote confirmation dialog.
     *
     * @private
     *
     * @return {@Ext.data.Store}
     */
    createPromotionProfileStore: function (records) {
        var self = this,
            data = [],
            provider,
            profileId,
            profileName;

        // FIXME: convoluted way to filter only profiles which repositories can be promoted into

        provider = records[0].get('provider');
        profileId = records[0].get('profileId');

        // TODO: Translate into name with profile store
        profileName = profileId;

        self.profileStore.each(function (rec) {
            // can only promote to groups
            if (rec.get('mode') !== 'GROUP') {
                return;
            }

            // group must match repository type
            if (rec.get('repositoryType') !== provider) {
                return;
            }

            // do not allow the user to promote to a group, if what they are
            // promoting is in the same group profile, and is only thing selected
            if (records.length === 1 && profileName === rec.get('name')) {
                return;
            }

            data.push([
                rec.get('id'),
                rec.get('name')
            ]);
        });

        return NX.create('Ext.data.ArrayStore', {
            id: 0,

            fields: [
                'id',
                'name'
            ],

            data: data
        });
    },

    /**
     * @private
     */
    displaySuccess: function (message) {
        Nexus.messages.show('Staging', message);

        this.repositoryStore.reload();
    },

    /**
     * @private
     */
    displayFailure: function (message, response) {
        Sonatype.utils.connectionError(response, message, false);

        this.repositoryStore.reload();
    }
});