/*
 * Copyright (c) 2008-2014 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * User-token settings view.
 *
 * @class Nexus.usertoken.SettingsView
 * @extends Ext.Panel
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.SettingsView', {
  extend: 'Ext.Panel',

  requires: [
    'Nexus.wonderland.AuthenticateButton',
    'Nexus.usertoken.Icons'
  ],
  requirejs: ['Nexus/usertoken'],
  mixins: [
    'Nexus.LogAwareMixin'
  ],

  constructor: function (config) {
    var self = this,
        icons = Nexus.usertoken.Icons;

    self.protectContentCheckbox = new Ext.form.Checkbox({
      name: 'protectContent',
      fieldLabel: 'Protect Content',
      helpText: 'Select this option to require that user tokens are used for repository authentication. This does not affect UI access.',
      disabled: true
    });

    self.settingsForm = new Ext.FormPanel({
      border: false,
      labelWidth: 100,
      layoutConfig: {
        labelSeparator: ''
      },
      items: [
        {
          xtype: 'checkbox',
          name: 'enabled',
          fieldLabel: 'Enabled',
          helpText: 'Enable user token functionality'
        },
        self.protectContentCheckbox
      ],
      buttons: [
        {
          text: 'Save',
          tooltip: 'Save settings',
          formBind: true,
          scope: self,
          handler: self.saveSettings
        },
        new Nexus.ext.LinkButton({
          text: 'Discard',
          tooltip: 'Discard changes',
          formBind: false,
          scope: self,
          handler: self.discardSettings
        })
      ]
    });

    // TODO: Show warning indicator if security is disabled and/or realm is not configured?

    // TODO: Show warning indicator if capability failed to activate

    self.resetPanel = new Ext.form.FieldSet({
      title: 'Reset',
      autoHeight: true,
      hidden: true,
      items: [
        {
          xtype: 'component',
          cls: 'nx-usertoken-icon',
          html: icons.get('error').variant('x32').img
        },
        {
          xtype: 'label',
          html: 'Reset will invalidate <span style="font-weight: bold;">ALL</span> existing user tokens and force new tokens to be created the next time accessed.<br/><br clear="both"/>',
          cls: 'nx-usertoken-message'
        },
        {
          xtype: 'nx-wonderland-button-authenticate',
          text: 'Reset All User Tokens',
          tooltip: 'Authenticate and reset all user tokens',
          listeners: {
            'authenticated': {
              scope: self,
              fn: self.resetAllUserTokens
            }
          }
        }
      ]
    });

    Ext.apply(self, {
      cls: 'nx-usertoken-SettingsView',
      border: false,
      layout: 'vbox',
      layoutConfig: {
        align: 'stretch'
      },
      items: [
        {
          xtype: 'fieldset',
          title: 'Settings',
          autoHeight: true,
          items: [
            self.settingsForm
          ]
        },

        self.resetPanel
      ]
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);

    // Load settings after layout, has to be done post superclass.constructor()
    self.on('afterlayout', self.loadSettings, self, {
      single: true
    });
  },

  getWaitMask: function () {
    var self = this;

    if (!self.waitMask) {
      self.waitMask = new Ext.LoadMask(self.getEl(), {
        msg: 'Please wait'
      });
    }
    return self.waitMask;
  },

  /**
   * @param {Boolean} enabled
   */
  setEnabled: function (enabled) {
    // Update enabled flag, this won't really help too much if the view is already opened
    Nexus.usertoken.enabled = enabled;

    // Toggle components visible
    this.resetPanel.setVisible(enabled);

    // Toggle components enabled
    if (enabled) {
      this.protectContentCheckbox.enable();
    }
    else {
      this.protectContentCheckbox.disable();
    }

    // need to re-layout because resetPanel will start hidden and initially has no size
    this.doLayout();
  },

  saveSettings: function () {
    var self = this, form;

    self.logDebug('Save settings');

    self.getWaitMask().show();

    form = self.settingsForm.getForm();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/settings'),
      method: 'PUT',
      scope: self,

      jsonData: {
        enabled: form.findField('enabled').getValue(),
        protectContent: form.findField('protectContent').getValue()
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('Settings saved');
        self.setEnabled(form.findField('enabled').getValue());
      }
    });
  },

  loadSettings: function () {
    var self = this;

    self.logDebug('Load settings');

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/settings'),
      method: 'GET',
      scope: self,

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function (response) {
        var settings = Ext.decode(response.responseText);
        self.settingsForm.getForm().setValues({
          enabled: settings.enabled,
          protectContent: settings.protectContent
        });
        self.logDebug('Settings loaded');

        self.setEnabled(settings.enabled);
      }
    });
  },

  discardSettings: function () {
    var self = this;

    self.logDebug('Discard settings');

    // reload settings
    self.loadSettings();
  },

  resetAllUserTokens: function (button, authTicket) {
    var self = this;

    self.logDebug('Reset all user tokens');

    self.getWaitMask().show();

    Ext.Ajax.request({
      url: Nexus.usertoken.resourcePath('/users'),
      method: 'DELETE',
      scope: self,

      headers: {
        'X-NX-AuthTicket': authTicket
      },

      callback: function () {
        self.getWaitMask().hide();
      },

      success: function () {
        Nexus.usertoken.showMessage('All user tokens have been reset');
      }
    });
  }

}, function () {
  var type = this;

  // Attach SettingsView to navigation panel if current user has appropriate permissions
  Sonatype.Events.addListener('nexusNavigationInit', function (panel) {
    var p = Sonatype.lib.Permissions;
    if (p.checkPermission('nexus:usertoken:settings', p.READ)) {
      panel.add({
        enabled: true,
        sectionId: 'st-nexus-security',
        title: Nexus.usertoken.TITLE,
        tabId: 'usertoken',
        tabTitle: Nexus.usertoken.TITLE,
        tabCode: type
      });
    }
  });
});
